/**
 * Copyright (c) 2017-2021 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecContext class declaration
 */

#ifndef LIBTEEC_TEEC_CONTEXT_H_
#define LIBTEEC_TEEC_CONTEXT_H_

#include "TeecSession.h"
#include "TeecSharedMemory.h"

#include <unordered_map>
#include <string>

#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#include <tee_client_api.h>

namespace extension {
namespace libteec {

using TeecSessionMap = std::unordered_map<std::string, TeecSessionPtr>;
using TeecSharedMemoryMap = std::unordered_map<std::string, TeecSharedMemoryPtr>;

class TeecContext final
{
public:
    TeecContext(const std::string& id);
    ~TeecContext();

    // returns session ID used to recognize sessions in GetSession and CloseSession
    std::string OpenSession(const std::string& uuidstr, uint32_t connectionMethod,
                            const void* connectionData, TEEC_Operation* operation,
                            uint32_t* returnOrigin);
    TeecSessionPtr GetSession(const std::string& id) const;
    void CloseSession(const std::string& id);

    // returns memory ID used to recognise the memory further down the road
    std::string CreateSharedMemory(size_t size, uint32_t flags);
    std::string CreateSharedMemory(void* buffer, size_t size, uint32_t flags);
    TeecSharedMemoryPtr GetSharedMemory(const std::string& memId) const;
    void ReleaseSharedMemory(const std::string& memId);

private:
    TEEC_Context mContext;
    TeecSessionMap mSessions;
    TeecSharedMemoryMap mSharedMemories;
    static const std::string teeFeatureName;
    bool teeIsEnabled();
};

using TeecContextPtr = std::shared_ptr<TeecContext>;

} // namespace libteec
} // namespace extension

#endif // LIBTEEC_TEEC_CONTEXT_H_
