/**
 * Copyright (c) 2017-2021 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecSharedMemory class declaration
 */

#ifndef LIBTEEC_TEEC_SHARED_MEMORY_H_
#define LIBTEEC_TEEC_SHARED_MEMORY_H_

#include <memory>
#include <vector>

#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#include <tee_client_api.h>

namespace extension {
namespace libteec {

using DataBuffer = std::vector<uint8_t>;

class TeecSharedMemory final
{
public:
    TeecSharedMemory(TEEC_Context* ctx, size_t size, uint32_t flags);
    TeecSharedMemory(TEEC_Context* ctx, void* buffer, size_t size, uint32_t flags);
    ~TeecSharedMemory();

    void SetData(const DataBuffer& data, size_t offset);
    void GetData(DataBuffer& data, size_t offset) const;

    TEEC_SharedMemory* GetMemoryPointer();
private:
    TEEC_SharedMemory mMemory;
};

using TeecSharedMemoryPtr = std::shared_ptr<TeecSharedMemory>;

} // namespace libteec
} // namespace extension

#endif // LIBTEEC_TEEC_SHARED_MEMORY_H_