/**
 * Copyright (c) 2017-2021 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  Teec to/from JSON translation function declarations
 */

#ifndef LIBTEEC_TEEC_TRANSLATIONS_H_
#define LIBTEEC_TEEC_TRANSLATIONS_H_

#include "TeecSharedMemory.h"

#include "TeecTempMemoryAllocator.h"

#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#include <tee_client_api.h>

#include <string>

#include <common/picojson.h>
#include <common/platform_exception.h>


namespace extension {
namespace libteec {
namespace translations {

std::string TeecResultToString(TEEC_Result result);
uint32_t StringToTeecLoginMethod(const std::string& str);
TEEC_Operation OperationFromJSON(const picojson::value& value, TeecTempMemoryAllocator& allocator);
DataBuffer DataBufferFromJSON(const picojson::value& value);
uint32_t SharedMemoryFlagsFromJSON(const picojson::value& value);
picojson::value UpdateJSONParams(const picojson::value& json, TEEC_Operation op);
picojson::value JSONFromDataBuffer(const DataBuffer& buffer);

} // namespace translations
} // namespace libteec
} // namespace extension

#endif // LIBTEEC_TEEC_TRANSLATIONS_H_
